"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseAgent = void 0;
const error_1 = require("../error");
const dids_1 = require("../modules/dids");
const generic_records_1 = require("../modules/generic-records");
const mdoc_1 = require("../modules/mdoc");
const sd_jwt_vc_1 = require("../modules/sd-jwt-vc");
const W3cCredentialsApi_1 = require("../modules/vc/W3cCredentialsApi");
const x509_1 = require("../modules/x509");
const storage_1 = require("../storage");
const UpdateAssistant_1 = require("../storage/migration/UpdateAssistant");
const wallet_1 = require("../wallet");
const error_2 = require("../wallet/error");
const AgentModules_1 = require("./AgentModules");
const EventEmitter_1 = require("./EventEmitter");
const context_1 = require("./context");
class BaseAgent {
    constructor(agentConfig, dependencyManager) {
        this._isInitialized = false;
        this.dependencyManager = dependencyManager;
        this.agentConfig = agentConfig;
        this.logger = this.agentConfig.logger;
        this.logger.info('Creating agent with config', {
            agentConfig: agentConfig.toJSON(),
        });
        if (!this.agentConfig.walletConfig) {
            this.logger.warn('Wallet config has not been set on the agent config. ' +
                'Make sure to initialize the wallet yourself before initializing the agent, ' +
                'or provide the required wallet configuration in the agent constructor');
        }
        // Resolve instances after everything is registered
        this.eventEmitter = this.dependencyManager.resolve(EventEmitter_1.EventEmitter);
        this.agentContext = this.dependencyManager.resolve(context_1.AgentContext);
        this.genericRecords = this.dependencyManager.resolve(generic_records_1.GenericRecordsApi);
        this.dids = this.dependencyManager.resolve(dids_1.DidsApi);
        this.wallet = this.dependencyManager.resolve(wallet_1.WalletApi);
        this.w3cCredentials = this.dependencyManager.resolve(W3cCredentialsApi_1.W3cCredentialsApi);
        this.sdJwtVc = this.dependencyManager.resolve(sd_jwt_vc_1.SdJwtVcApi);
        this.x509 = this.dependencyManager.resolve(x509_1.X509Api);
        this.mdoc = this.dependencyManager.resolve(mdoc_1.MdocApi);
        const defaultApis = [
            this.genericRecords,
            this.dids,
            this.wallet,
            this.w3cCredentials,
            this.sdJwtVc,
            this.x509,
            this.mdoc,
        ];
        // Set the api of the registered modules on the agent, excluding the default apis
        this.modules = (0, AgentModules_1.getAgentApi)(this.dependencyManager, defaultApis);
    }
    get isInitialized() {
        return this._isInitialized && this.wallet.isInitialized;
    }
    async initialize() {
        const { walletConfig } = this.agentConfig;
        if (this._isInitialized) {
            throw new error_1.CredoError('Agent already initialized. Currently it is not supported to re-initialize an already initialized agent.');
        }
        if (!this.wallet.isInitialized && walletConfig) {
            await this.wallet.initialize(walletConfig);
        }
        else if (!this.wallet.isInitialized) {
            throw new error_2.WalletError('Wallet config has not been set on the agent config. ' +
                'Make sure to initialize the wallet yourself before initializing the agent, ' +
                'or provide the required wallet configuration in the agent constructor');
        }
        // Make sure the storage is up to date
        const storageUpdateService = this.dependencyManager.resolve(storage_1.StorageUpdateService);
        const isStorageUpToDate = await storageUpdateService.isUpToDate(this.agentContext);
        this.logger.info(`Agent storage is ${isStorageUpToDate ? '' : 'not '}up to date.`);
        if (!isStorageUpToDate && this.agentConfig.autoUpdateStorageOnStartup) {
            const updateAssistant = new UpdateAssistant_1.UpdateAssistant(this);
            await updateAssistant.initialize();
            await updateAssistant.update({ backupBeforeStorageUpdate: this.agentConfig.backupBeforeStorageUpdate });
        }
        else if (!isStorageUpToDate) {
            const currentVersion = await storageUpdateService.getCurrentStorageVersion(this.agentContext);
            // Close wallet to prevent un-initialized agent with initialized wallet
            await this.wallet.close();
            throw new error_1.CredoError(
            // TODO: add link to where documentation on how to update can be found.
            `Current agent storage is not up to date. ` +
                `To prevent the framework state from getting corrupted the agent initialization is aborted. ` +
                `Make sure to update the agent storage (currently at ${currentVersion}) to the latest version (${UpdateAssistant_1.UpdateAssistant.frameworkStorageVersion}). ` +
                `You can also downgrade your version of Credo.`);
        }
    }
    get config() {
        return this.agentConfig;
    }
    get context() {
        return this.agentContext;
    }
}
exports.BaseAgent = BaseAgent;
//# sourceMappingURL=BaseAgent.js.map