"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.P521Jwk = void 0;
exports.isValidP521JwkPublicKey = isValidP521JwkPublicKey;
const ec_compression_1 = require("ec-compression");
const error_1 = require("../../../error");
const utils_1 = require("../../../utils");
const KeyType_1 = require("../../KeyType");
const jwa_1 = require("../jwa");
const alg_1 = require("../jwa/alg");
const Jwk_1 = require("./Jwk");
const validate_1 = require("./validate");
class P521Jwk extends Jwk_1.Jwk {
    constructor({ x, y }) {
        super();
        const xAsBytes = typeof x === 'string' ? Uint8Array.from(utils_1.TypedArrayEncoder.fromBase64(x)) : x;
        const yAsBytes = typeof y === 'string' ? Uint8Array.from(utils_1.TypedArrayEncoder.fromBase64(y)) : y;
        this.affinePoint = new ec_compression_1.AffinePoint(xAsBytes, yAsBytes);
    }
    get kty() {
        return jwa_1.JwaKeyType.EC;
    }
    get crv() {
        return jwa_1.JwaCurve.P521;
    }
    get keyType() {
        return P521Jwk.keyType;
    }
    get supportedEncryptionAlgorithms() {
        return P521Jwk.supportedEncryptionAlgorithms;
    }
    get supportedSignatureAlgorithms() {
        return P521Jwk.supportedSignatureAlgorithms;
    }
    get x() {
        return utils_1.TypedArrayEncoder.toBase64URL(this.affinePoint.xBytes);
    }
    get y() {
        return utils_1.TypedArrayEncoder.toBase64URL(this.affinePoint.yBytes);
    }
    /**
     * Returns the uncompressed public key of the P-521 JWK.
     */
    get publicKey() {
        return this.affinePoint.decompressedForm;
    }
    /**
     * Returns the compressed public key of the P-521 JWK.
     */
    get publicKeyCompressed() {
        return this.affinePoint.compressedForm;
    }
    toJson() {
        return {
            ...super.toJson(),
            crv: this.crv,
            x: this.x,
            y: this.y,
        };
    }
    static fromJson(jwk) {
        if (!isValidP521JwkPublicKey(jwk)) {
            throw new Error("Invalid 'P-521' JWK.");
        }
        return new P521Jwk({
            x: jwk.x,
            y: jwk.y,
        });
    }
    static fromPublicKey(publicKey) {
        if ((0, ec_compression_1.isValidCompressedPublicKeyFormat)(publicKey, ec_compression_1.Secp521r1)) {
            const affinePoint = ec_compression_1.AffinePoint.fromCompressedPoint(publicKey, ec_compression_1.Secp521r1);
            return new P521Jwk({ x: affinePoint.xBytes, y: affinePoint.yBytes });
        }
        if ((0, ec_compression_1.isValidDecompressedPublicKeyFormat)(publicKey, ec_compression_1.Secp521r1)) {
            const affinePoint = ec_compression_1.AffinePoint.fromDecompressedPoint(publicKey, ec_compression_1.Secp521r1);
            return new P521Jwk({ x: affinePoint.xBytes, y: affinePoint.yBytes });
        }
        throw new error_1.CredoError(`${this.keyType} public key is neither a valid compressed or uncompressed key. Key prefix '${publicKey[0]}', key length '${publicKey.length}'`);
    }
}
exports.P521Jwk = P521Jwk;
P521Jwk.supportedEncryptionAlgorithms = [];
P521Jwk.supportedSignatureAlgorithms = [alg_1.JwaSignatureAlgorithm.ES512];
P521Jwk.keyType = KeyType_1.KeyType.P521;
function isValidP521JwkPublicKey(jwk) {
    return ((0, validate_1.hasKty)(jwk, jwa_1.JwaKeyType.EC) &&
        (0, validate_1.hasCrv)(jwk, jwa_1.JwaCurve.P521) &&
        (0, validate_1.hasX)(jwk) &&
        (0, validate_1.hasY)(jwk) &&
        (0, validate_1.hasValidUse)(jwk, {
            supportsEncrypting: true,
            supportsSigning: true,
        }));
}
//# sourceMappingURL=P521Jwk.js.map